﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;



namespace MVB_RestBookService
{
    public partial class IndexFrame : Form
    {
        //Member Variables
        private System.Timers.Timer timerRequest;
        private System.Timers.Timer timerRefreshForm;

        private bool requestInProgress;
        private bool requestNeeded;
        private bool refreshNeeded;
        private JsonDataContainerIndex jsonDataContainerIndex;

        private string accessToken;
        private string index;
        private TextBox textBox;



        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="accessToken">The AccessToken used to perform Index Rest-Requests.</param>
        /// <param name="index">Index to search, e.g. 'author', 'title' or 'identifier'.</param>
        /// <param name="textBox">Textbox the selected Index Item will be inserted into.</param>
        /// <param name="formTitle">Title for this Form.</param>
        public IndexFrame(string accessToken, string index, TextBox textBox, string formTitle)
        {
            InitializeComponent();

            this.accessToken = accessToken;
            this.index = index;
            this.textBox = textBox;
            this.Text = formTitle;

            //Timer for Index Rest-Requests
            timerRequest = new System.Timers.Timer();
            timerRequest.Interval = 2000;
            timerRequest.Elapsed += timerRequest_Elapsed;
            timerRequest.Start();

            //Timer for updating the Frame (may not be the Rest-Request Timer, since it is running on a different Thread than the Form, which causes Exceptions)
            timerRefreshForm = new System.Timers.Timer();
            timerRefreshForm.SynchronizingObject = this;
            timerRefreshForm.Interval = 500;
            timerRefreshForm.Elapsed += timerRefreshForm_Elapsed;
            timerRefreshForm.Start();

            //Listeners
            FormClosing += IndexFrame_FormClosing;
        }



        /// <summary>
        /// Listener for Index Rest-Requests - will perform an Index Update, when the timerRequest runs out.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timerRequest_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            //Request in Progress
            if (requestInProgress)
            {
                return;
            }

            //Perform new Index Search, if needed
            if (!index_waterMarkTextBox_search.Text.Equals("") && requestNeeded)
            {
                requestInProgress = true;
                requestNeeded = false;

                //Variables
                RestClient client;
                HttpWebResponse json;
                string responseAsString = "";

                //Config Request
                client = new RestClient();
                client.EndPoint = @"http://api.vlb.de/api/v1/index/" + index + @"/" + index_waterMarkTextBox_search.Text;
                client.AccessToken = string.Format("Bearer {0}", accessToken);
                client.ContentTypeString = "application/json";
                client.Method = HttpVerb.GET;

                //Send Request
                try
                {
                    json = client.MakeRequest("", ref responseAsString);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Ein Fehler ist aufgetreten:\n\n" + ex.Message, "Fehler");
                    requestInProgress = false;
                    return;
                }

                //Patchwork to make the Json Deserializer work
                responseAsString = "{ \"content\": " + responseAsString + " }";

                //Init Json.net Deserializer
                JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
                jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
                jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

                //Create a Container, containing all Json Fields and deserialize Json Data
                jsonDataContainerIndex = JsonConvert.DeserializeObject<JsonDataContainerIndex>(responseAsString, jsonSerializerSettings);

                //Request done
                requestInProgress = false;
                refreshNeeded = true;
            }

        }



        /// <summary>
        /// Listener for Frame refresh - will update the Frame with current Rest-Request Data, when the timerRefreshForm runs out.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timerRefreshForm_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            //Request in Progress, don't refresh
            if (requestInProgress)
            {
                return;
            }

            //No Data available
            if (jsonDataContainerIndex == null)
            {
                return;
            }

            //Refresh needed
            if (refreshNeeded)
            {
                //Clear DataGridView
                index_dataGridView.Rows.Clear();

                //Add Data
                for (int i = 0; i < jsonDataContainerIndex.content.Length; i++)
                {
                    index_dataGridView.Rows.Add();
                    index_dataGridView[0, i].Value = jsonDataContainerIndex.content[i].value;
                }

                //Refresh done
                refreshNeeded = false;
            }
        }



        /// <summary>
        /// Listener for the onClosing Event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void IndexFrame_FormClosing(Object sender, FormClosingEventArgs e)
        {
            timerRequest.Stop();
            timerRequest.Dispose();
            timerRefreshForm.Stop();
            timerRefreshForm.Dispose();
        }



        /// <summary>
        /// User changed Search Text, a new Rest-Request is needed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void waterMarkTextBox_search_TextChanged(object sender, EventArgs e)
        {
            timerRequest.Stop();
            timerRequest.Start();
            requestNeeded = true;
        }



        /// <summary>
        /// Fired, if the user double-clicks at a title in the index_dataGridView.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void index_dataGridView_onDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if(index_dataGridView.SelectedRows.Count > 0)
            {
                textBox.Text = index_dataGridView.SelectedRows[0].Cells[0].Value.ToString();
                this.Close();
            }
        }
    }
}
